package de.intevation.flys.exports;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.DefaultXYDataset;

import de.intevation.artifacts.Artifact;

import de.intevation.flys.model.DischargeTable;
import de.intevation.flys.model.Gauge;
import de.intevation.flys.model.River;
import de.intevation.flys.model.TimeInterval;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.DischargeTables;
import de.intevation.flys.artifacts.model.GaugesFactory;


/**
 * An OutGenerator that generates discharge curves.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class DischargeCurveGenerator extends XYChartGenerator {

    /** The logger used in this generator.*/
    private static Logger logger =
        Logger.getLogger(DischargeCurveGenerator.class);


    public static final String I18N_CHART_TITLE =
        "chart.discharge.curve.title";

    public static final String I18N_CHART_SUBTITLE =
        "chart.discharge.curve.subtitle";

    public static final String I18N_XAXIS_LABEL =
        "chart.discharge.curve.xaxis.label";

    public static final String I18N_YAXIS_LABEL =
        "chart.discharge.curve.yaxis.label";

    public static final String I18N_CHART_TITLE_DEFAULT  = "Abflusskurven";
    public static final String I18N_XAXIS_LABEL_DEFAULT  = "Q [m\u00b3/s]";
    public static final String I18N_YAXIS_LABEL_DEFAULT  = "W [cm]";


    /** The Dataset that contains the data for the chart.*/
    protected DefaultXYDataset dataset;


    public DischargeCurveGenerator() {
        super();

        this.dataset = new DefaultXYDataset();
    }


    protected String getChartTitle() {
        return msg(I18N_CHART_TITLE, I18N_CHART_TITLE_DEFAULT);
    }


    @Override
    protected void addSubtitles(JFreeChart chart) {

    }


    protected String getXAxisLabel() {
        return msg(I18N_XAXIS_LABEL, I18N_XAXIS_LABEL_DEFAULT);
    }


    protected String getYAxisLabel() {
        return msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT);
    }


    protected void addDatasets(JFreeChart chart) {
        XYPlot plot = (XYPlot) chart.getPlot();
        plot.setDataset(0, dataset);
    }


    public void doOut(Artifact artifact, String facet, Document attr) {
        logger.debug("DischargeCurveGenerator.doOut: " + facet);

        if (!(artifact instanceof FLYSArtifact)) {
            logger.error("Artifact is no instance of FLYSArtifact.");
            return;
        }

        FLYSArtifact flysArtifact = (FLYSArtifact) artifact;

        River    river    = flysArtifact.getRiver();
        double[] distance = flysArtifact.getDistance();

        List<double[]> ranges = new ArrayList<double[]>();
        ranges.add(distance);

        List<Gauge> gauges   = GaugesFactory.getGauges(river);
        List<Gauge> filtered = GaugesFactory.filterRanges(gauges, ranges);

        if (logger.isDebugEnabled()) {
            int numGauges   = gauges != null ? gauges.size() : 0;
            int numFiltered = filtered != null ? filtered.size() : 0;

            logger.debug("++++++++++++++++++++");
            logger.debug("Search gauges for river: " + river.getName());
            logger.debug("-> ... from range: " + distance[0]);
            logger.debug("-> ... to range: " + distance[1]);
            logger.debug("-> Found " + numGauges + " gauges in total.");
            logger.debug("-> Found " + numFiltered + " gauges after filtering.");
            logger.debug("++++++++++++++++++++");
        }

        String[] gaugeNames = new String[filtered.size()];
        int idx = 0;
        for (Gauge gauge: filtered) {
            gaugeNames[idx++] = gauge.getName();
        }

        DischargeTables dt = new DischargeTables(river.getName(), gaugeNames);
        Map<String, double[][]> gaugeValues = dt.getValues(100d);

        for (Gauge gauge: filtered) {
            String gaugeName = gauge.getName();

            double[][] values = gaugeValues.get(gaugeName);
            dataset.addSeries(getSeriesName(gauge), values);
        }
    }


    protected String getSeriesName(Gauge gauge) {
        List<DischargeTable> dts = gauge.getDischargeTables();

        for (DischargeTable dt: dts) {
            if (dt.getKind() == 0) {
                TimeInterval ti = dt.getTimeInterval();

                Date start = ti.getStartTime();
                Date end   = ti.getStopTime();

                String name  = gauge.getName();

                if (end == null) {
                    Object[] args = new Object[] { name, start };
                    return msg(
                        "chart.discharge.curve.curve.valid.from",
                        "",
                        args);
                }
                else {
                    Object[] args = new Object[] { name, start, end };
                    return msg(
                        "chart.discharge.curve.curve.valid.range",
                        "",
                        args);
                }
            }
        }

        return gauge.getName();
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
